require "import"
import "com.androlua.*"
import "android.widget.*"
import "android.view.*"
import "android.app.*"
import "android.content.*"
import "android.content.pm.*"
import "android.net.Uri"
import "java.builder.*"
import "java.io.File"
import "android.os.*"

-- Constants
local downloadDirectory = "/sdcard/Download/"
local url = "http://d.jieshuo.ltd/?f=beta&t=%E6%B5%8B%E8%AF%95%E7%89%88%E4%B8%8B%E8%BD%BD"

local layout
local downloadButton
local installButton
local shareButton
local deleteButton

local downloadLink
local downloadManager
local downloadId
local progressBar
local versionRelease = ""
local progressBarProgress = 0
local handler = luajava.newInstance("android.os.Handler")

-- Function to display a Toast message
local function showToast(message)
    activity.runOnUiThread(function()
        Toast.makeText(activity, message, Toast.LENGTH_SHORT).show()
    end)
end

-- Function to update the progress bar
local function updateProgressBar()
    if progressBarProgress < 100 then
        local bytesDownloaded = 0
        local bytesTotal = 0
        local query = DownloadManager.Query()
        query.setFilterById({downloadId}) -- Wrap downloadId in a table

        local cursor = downloadManager.query(query)
        if cursor and cursor.moveToFirst() then
            bytesDownloaded = cursor.getLong(cursor.getColumnIndex(DownloadManager.COLUMN_BYTES_DOWNLOADED_SO_FAR))
            bytesTotal = cursor.getLong(cursor.getColumnIndex(DownloadManager.COLUMN_TOTAL_SIZE_BYTES))
        end
        cursor.close()

        if bytesTotal > 0 then
            progressBarProgress = math.floor((bytesDownloaded / bytesTotal) * 100)
            activity.runOnUiThread(function()
                progressBar.setProgress(progressBarProgress)
            end)
        end

        handler.postDelayed(updateProgressBar, 1000) -- Update every second (adjust as needed)
    else
        activity.runOnUiThread(function()
            progressBar.setVisibility(View.GONE)
            installButton.setVisibility(View.VISIBLE)  -- Show the install button
            shareButton.setVisibility(View.VISIBLE)   -- Show the share button
            deleteButton.setVisibility(View.VISIBLE)  -- Show the delete button
        end)
    end
end

-- Function to cancel the download
local function cancelDownload()
    if downloadId then
        -- Use a BroadcastReceiver to cancel the download
        local receiver = BroadcastReceiver{
            onReceive = function(context, intent)
                local receivedDownloadId = intent.getLongExtra(DownloadManager.EXTRA_DOWNLOAD_ID, -1)
                if receivedDownloadId == downloadId then
                    downloadManager.remove({downloadId})
                    progressBar.setVisibility(View.GONE)
                    showToast("your download has been cancel.")
                    activity.unregisterReceiver(receiver) -- Unregister the receiver after canceling
                end
            end
        }

        local filter = Intent(DownloadManager.ACTION_DOWNLOAD_COMPLETE)
        activity.registerReceiver(receiver, filter)

        -- Cancel the download via broadcast
        local cancelIntent = Intent("android.intent.action.DOWNLOAD_CANCEL")
        cancelIntent.putExtra(DownloadManager.EXTRA_DOWNLOAD_ID, downloadId)
        activity.sendBroadcast(cancelIntent)
    end
end

-- Function to retry the download
local function retryDownload()
    if downloadLink then
        if not checkIfFileExists() then
            downloadManager = activity.getSystemService(Context.DOWNLOAD_SERVICE)
            local request = DownloadManager.Request(Uri.parse(downloadLink))
            request.setTitle("download update")
request.setDescription("Download application update")
            request.setDestinationInExternalPublicDir(Environment.DIRECTORY_DOWNLOADS, "app_update_" .. versionRelease .. ".apk")
            request.setNotificationVisibility(DownloadManager.Request.VISIBILITY_VISIBLE_NOTIFY_COMPLETED)

            downloadId = downloadManager.enqueue(request)
downloadButton.setText("Cancel download")
            updateProgressBar()
showToast("The update is downloading in the background.")
        end
    else
showToast("The download link cannot be found.")
    end
end

-- Function to finish the task
local function finishTask()
    -- Implement the action to be taken when the "Finish Task" button is clicked.
    activity.finish() -- This will close the current activity.
    -- Add your additional logic here if needed.
end

-- Function to handle sharing the downloaded file
local function shareDownloadedFile()
    local filePath = downloadDirectory .. "app_update_" .. versionRelease .. ".apk"
    local file = File(filePath)
    if file.exists() and file.isFile() then
        local shareIntent = Intent()
        shareIntent.setAction(Intent.ACTION_SEND)
        shareIntent.putExtra(Intent.EXTRA_STREAM, activity.getUriForFile(file))
        shareIntent.setType("application/vnd.android.package-archive")
        activity.startActivity(Intent.createChooser(shareIntent, "Share Update File"))
    else
        showToast("File not found for sharing.")
    end
end

-- Function to handle deleting the downloaded file with confirmation dialog
local function deleteDownloadedFile()
    local filePath = downloadDirectory .. "app_update_" .. versionRelease .. ".apk"
    local file = File(filePath)
    if file.exists() and file.isFile() then
        local builder = AlertDialog.Builder(activity)
builder.setTitle("Confirm deletion")
builder.setMessage("Do you really want to delete the update file?")

        builder.setPositiveButton("yes", DialogInterface.OnClickListener{
            onClick = function(dialog, which)
                -- Implement the logic to delete the file
                file.delete()
showToast("The update file has been deleted.")
                deleteButton.setVisibility(View.INVISIBLE) -- Hide the delete button after deletion
                shareButton.setVisibility(View.INVISIBLE)   -- Hide the share button after deletion
                installButton.setVisibility(View.INVISIBLE) -- Hide the install button after deletion
            end
        })

        builder.setNegativeButton("no", DialogInterface.OnClickListener{
            onClick = function(dialog, which)
                -- User clicked "No," do nothing
            end
        })

        builder.show()
    else
showToast("The file was not found for deletion.")
    end
end

-- Function to install the downloaded update using FileProvider
local function installDownloadedUpdate()
    local filePath = downloadDirectory .. "app_update_" .. versionRelease .. ".apk"
    local file = File(filePath)
    if file.exists() and file.isFile() then
        local uri = activity.getUriForFile(file)

        local intent = Intent(Intent.ACTION_VIEW)
        intent.setDataAndType(uri, "application/vnd.android.package-archive")
        intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)

        activity.startActivity(intent)
    else
        showToast("File not found for installation.")
    end
end

-- Check if the downloaded file exists
local function checkIfFileExists()
    local filePath = downloadDirectory .. "app_update_" .. versionRelease .. ".apk"
    local file = File(filePath)
    if file.exists() and file.isFile() then
        local builder = AlertDialog.Builder(activity)
        builder.setTitle("File Exists")
        builder.setMessage("The update file already exists. Do you want to install it, share it, or delete it?")

        builder.setPositiveButton("Install", DialogInterface.OnClickListener{
            onClick = function(dialog, which)
                installDownloadedUpdate()
                installButton.setVisibility(View.INVISIBLE) -- Hide the install button after installation
                shareButton.setVisibility(View.INVISIBLE)   -- Hide the share button after installation
                deleteButton.setVisibility(View.INVISIBLE)  -- Hide the delete button after installation
            end
        })

        builder.setNeutralButton("Share", DialogInterface.OnClickListener{
            onClick = function(dialog, which)
                shareDownloadedFile()
            end
        })

        builder.setNegativeButton("Delete", DialogInterface.OnClickListener{
            onClick = function(dialog, which)
                deleteDownloadedFile()
            end
        })

        builder.show()
        return true -- File exists
    end
    return false -- File doesn't exist
end

-- Function to start the update process
local function startUpdateProcess()
    Http.get(url, function(status, data)
        if status == 200 then
            local av = data:match("%d+%d+%d+%d+")
            local cv = activity.getPackageManager().getPackageInfo(activity.getPackageName(), 0).versionName

            if av and tonumber(av) > tonumber(cv) then
                downloadLink = data:match("(https?://[%w-_%.%?%.:/%=%&]+%.apk)")
                versionRelease = "v" .. av

                local releaseDate = data:match("rDate%s*=%s*'([^']+)'")

local message = "New version available: ".. av
                if releaseDate then
                    local sdf = SimpleDateFormat("yyyy-MM-dd HH:mm:ss")
                    local releaseDateObj = sdf.parse(releaseDate)
                    local formattedDate = SimpleDateFormat("MMMM dd, yyyy").format(releaseDateObj)
message = message .. "\nRelease date: " .. formattedDate
                end

                activity.runOnUiThread(function()
                    layout = LinearLayout(activity)
                    layout.setOrientation(LinearLayout.VERTICAL)

                    local title = TextView(activity)
title.setText("Update Available")
                    title.setTextSize(20)
                    title.setPadding(20, 20, 20, 20)
                    layout.addView(title)

                    local messageTextView = TextView(activity)
                    messageTextView.setText(message)
                    messageTextView.setPadding(20, 0, 20, 20)
                    layout.addView(messageTextView)

                    progressBar = ProgressBar(activity, nil, android.R.attr.progressBarStyleHorizontal)
                    progressBar.setMax(100)
                    progressBar.setVisibility(View.VISIBLE)
                    layout.addView(progressBar)

                    downloadButton = Button(activity)
downloadButton.setText("Download")
                    downloadButton.setOnClickListener(View.OnClickListener{
                        onClick = function(v)
                            if downloadLink then
                                if not checkIfFileExists() then
                                    downloadManager = activity.getSystemService(Context.DOWNLOAD_SERVICE)
                                    local request = DownloadManager.Request(Uri.parse(downloadLink))
request.setTitle("Download update")
request.setDescription("Download application update")
                                    request.setDestinationInExternalPublicDir(Environment.DIRECTORY_DOWNLOADS, "app_update_" .. versionRelease .. ".apk")
                                    request.setNotificationVisibility(DownloadManager.Request.VISIBILITY_VISIBLE_NOTIFY_COMPLETED)
                                    downloadId = downloadManager.enqueue(request)
downloadButton.setText("Cancel download")
                                    updateProgressBar()
showToast("The update is downloading in the background.")
                                end
                            else
showToast("The download link cannot be found.")
                            end
                        end
                    })
                    layout.addView(downloadButton)

                    installButton = Button(activity)
installButton.setText("Install")
                    installButton.setVisibility(View.INVISIBLE)  -- Initially, hide the install button
                    installButton.setOnClickListener(View.OnClickListener{
                        onClick = function(v)
                            installDownloadedUpdate()
                            installButton.setVisibility(View.INVISIBLE) -- Hide the install button after installation
                            shareButton.setVisibility(View.INVISIBLE)   -- Hide the share button after installation
                            deleteButton.setVisibility(View.INVISIBLE)  -- Hide the delete button after installation
                        end
                    })
                    layout.addView(installButton)

                    shareButton = Button(activity)
shareButton.setText("Share")
                    shareButton.setVisibility(View.INVISIBLE)   -- Initially, hide the share button
                    shareButton.setOnClickListener(View.OnClickListener{
                        onClick = function(v)
                            shareDownloadedFile()
                        end
                    })
                    layout.addView(shareButton)

                    deleteButton = Button(activity)
deleteButton.setText("delete")
                    deleteButton.setVisibility(View.INVISIBLE)  -- Initially, hide the delete button
                    deleteButton.setOnClickListener(View.OnClickListener{
                        onClick = function(v)
                            deleteDownloadedFile()
                        end
                    })
                    layout.addView(deleteButton)

                    local cancelButton = Button(activity)
cancelButton.setText("Cancel")
                    cancelButton.setOnClickListener(View.OnClickListener{
                        onClick = function(v)
                            cancelDownload()
                        end
                    })
                    layout.addView(cancelButton)

                    -- Create and add the "Finish Task" button
                    local finishTaskButton = Button(activity)
finishTaskButton.setText("End Task")
                    finishTaskButton.setOnClickListener(View.OnClickListener{
                        onClick = function(v)
                            finishTask()
                        end
                    })
                    layout.addView(finishTaskButton)

                    local dialog = AlertDialog.Builder(activity)
                    dialog.setView(layout)
                    dialog.setCancelable(false)
                    dialog.show()
                end)
            else
showToast("No updates!")
            end
        else
showToast("Connection error :-(")
        end
    end)
end

-- Call the startUpdateProcess function to initiate the update
startUpdateProcess()